=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Designed Jan. 2009 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:  Lib6G6.rb
# Original Date	:  03 Dec 2008 - version 3.0
# Type			:  Script library part of the LibFredo6 shared libraries
# Description	:  Contains some generic geometric methods
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module G6

#Check if a group is unique
def G6.is_group_unique?(g)
	entity = g.entities.find { |e| e.class != Sketchup::Group && e.class != Sketchup::ComponentInstance }
	return true unless entity.respond_to?(:parent)
	definition = entity.parent
	return true unless definition
	definition.count_instances == 1
end

#return the entities, either for a Group or for a component
def G6.grouponent_entities(entity)
	return Sketchup.active_model.active_entities unless entity
	(entity.class == Sketchup::ComponentInstance) ? entity.definition.entities : entity.entities
end

#Return the Component or Group instance currently open (or nil if at top level)
def G6.which_instance_opened
	model = Sketchup.active_model
	ee = model.active_entities
	mm = ee.parent
	
	#Current context is at the toplevel
	return nil unless mm.class == Sketchup::ComponentDefinition
	
	#Getting the instances and finding the one that has a transformation equal to Identity
	lsti = mm.instances
	return lsti[0] if lsti.length == 1
	lsti.each do |instance|
		center = instance.bounds.center
		return instance if instance.transformation * center == center
	end
	nil
end

#Map the model.start_operation method, managing the optional optimization last parameter in SU7
def G6.start_operation(model, title, speedup=false, *args)
	(SU_MAJOR_VERSION < 7) ? model.start_operation(title) : model.start_operation(title, speedup, *args)
end

def G6.continue_operation(model, title, speedup=false, *args)
	model.start_operation(title, speedup, *args) if (SU_MAJOR_VERSION >= 7)
end

#Compute the wireframe for a list of entities
#return a flat list of pairs of points to be usually drawn by view.draw GL_LINES
def G6.wireframe_entities(entities, hsh_edgeID=nil)
	G6._aux_wireframe_entities entities, Geom::Transformation.new, {}, [], 'model', hsh_edgeID
end

def G6._aux_wireframe_entities(entities, t, hedges, lcomp, parent, hsh_edgeID=nil)
	return [] unless entities
	entities.each do |e|
		if e.class == Sketchup::Edge
			next if hedges[e.to_s] == parent
			hedges[e.to_s] == parent
			lcomp.push t * e.start.position, t * e.end.position
			if hsh_edgeID
				hsh_edgeID[e.entityID] = true
				hsh_edgeID[e.start.entityID] = true
				hsh_edgeID[e.end.entityID] = true
			end	
		elsif e.class == Sketchup::Face
			e.edges.each do |edge|
				next if hedges[edge.to_s] == parent
				hedges[edge.to_s] = parent
				lcomp.push t * edge.start.position, t * edge.end.position
			end	
		elsif e.class == Sketchup::Group
			G6._aux_wireframe_entities e.entities, t * e.transformation, hedges, lcomp, e, hsh_edgeID	
		elsif e.class == Sketchup::ComponentInstance
			G6._aux_wireframe_entities e.definition.entities, t * e.transformation, hedges, lcomp, e, hsh_edgeID	
		end	
	end
	lcomp	
end

#Transform a vector. The method accounts for non orthogonal axes, as often the case when shear was applied
def G6.transform_vector(vector, t)
	return vector unless vector && vector.valid? && t
	axdt = vector.axes.collect { |v| t * v }
	vec = axdt[0] * axdt[1]
	vec.length = vector.length
	vec
end

#Check whether inference can apply, given a hash table of Entity Ids
def G6.not_auto_inference?(ip, hsh_entID)
	return true unless hsh_entID
	e = ip.vertex
	v = ip.edge
	f = ip.face
	return false if (e && hsh_entID[e.entityID]) || (v && hsh_entID[v.entityID]) ||
	                (f && hsh_entID[f.entityID])
	true
end

#Determine if an inference is a true one (close to vertex) or a remote one
def G6.true_inference_vertex?(view, ip, x, y)
	vertex = ip.vertex
	return false unless vertex
	pt2d = view.screen_coords ip.transformation * vertex.position
	return false if (pt2d.x - x).abs > 10 || (pt2d.y - y).abs > 10
	true
end

#---------------------------------------------------------------------------------------------------------------------------------
# Edges Properties
#---------------------------------------------------------------------------------------------------------------------------------

#Dialog box for Dicing parameters - Return a list [dice, dice_format]
def G6.ask_edge_prop_filter(titletool, edge_prop)
	#Creating the dialog box
	hparams = {}
	title = ((titletool) ? titletool + ' - ' : '') + T6[:T_DLG_EdgeProp_Title]
	dlg = Traductor::DialogBox.new title
		
	enum_yesno = { 'Y' => T6[:T_DLG_YES], 'N' => T6[:T_DLG_NO] }
	ted = ' '
	dlg.field_enum "Plain", ted + T6[:T_DLG_EdgePlain], 'Y', enum_yesno
	dlg.field_enum "Soft", ted + T6[:T_DLG_EdgeSoft], 'Y', enum_yesno
	dlg.field_enum "Smooth", ted + T6[:T_DLG_EdgeSmooth], 'Y', enum_yesno
	dlg.field_enum "Hidden", ted + T6[:T_DLG_EdgeHidden], 'N', enum_yesno

	#Invoking the dialog box
	hparams["Plain"] = (edge_prop =~ /P/i) ? 'Y' : 'N'
	hparams["Soft"] = (edge_prop =~ /S/i) ? 'Y' : 'N'
	hparams["Smooth"] = (edge_prop =~ /M/i) ? 'Y' : 'N'
	hparams["Hidden"] = (edge_prop =~ /H/i) ? 'Y' : 'N'
	
	#Cancel dialog box
	return edge_prop unless dlg.show! hparams		
	
	#Transfering the parameters
	ep = []
	ep.push 'P' if hparams["Plain"] == 'Y'
	ep.push 'S' if hparams["Soft"] == 'Y'
	ep.push 'M' if hparams["Smooth"] == 'Y'
	ep.push 'H' if hparams["Hidden"] == 'Y'
	ep.join ';;'	
end

#check the filtering by edge property
def G6.edge_filter?(edge, filter)
	filter = 'P;;S;;M;;H' unless filter && filter.length > 0
	
	return false if edge.smooth? && !(filter =~ /M/i)
	return false if edge.soft? && !(filter =~ /S/i)
	return false if edge.hidden? && !(filter =~ /H/i)
	return false if !(edge.smooth? || edge.soft? || edge.hidden?) && !(filter =~ /P/i)
	true
end

#check the filtering by edge property
def G6.edge_filter_text(filter)
	return T6[:T_DLG_EdgeAll] unless filter && filter.length > 0
	return T6[:T_DLG_EdgeAll] if filter =~ /P/i && filter =~ /S/i && filter =~ /M/i && filter =~ /H/i
	
	ls = []
	ls.push  T6[:T_DLG_EdgePlain] if filter =~ /P/i
	ls.push  T6[:T_DLG_EdgeSoft] if filter =~ /S/i
	ls.push  T6[:T_DLG_EdgeSmooth] if filter =~ /M/i
	ls.push  T6[:T_DLG_EdgeHidden] if filter =~ /H/i
	
	ls.join '+'
end

def G6.edge_prop_check_plain(prop)
	if prop == nil || prop == ''
		return 'P'
	elsif prop =~ /S|M|H/i
		lst = []
		lst.push 'S' if prop =~ /S/i
		lst.push 'M' if prop =~ /M/i
		lst.push 'H' if prop =~ /H/i
		return lst.join(';;')
	end	
	prop
end

#---------------------------------------------------------------------------------------------------------------------------------
# Geometry Utilities
#---------------------------------------------------------------------------------------------------------------------------------

#Compute a plane from 3 points
def G6.plane_by_3_points(pt1, pt2, pt3)
	vec1 = pt1.vector_to pt2
	vec2 = pt1.vector_to pt3
	[pt1, vec1 * vec2]
end

#calculate the normal to an edge of a face pointing toward the inside
def G6.normal_in_to_edge(edge, face)
	pt1 = edge.start.position
	pt2 = edge.end.position
	vec = face.normal * pt1.vector_to(pt2)
	vec.length = 1.0
	edge.reversed_in?(face) ? vec.reverse : vec
end

#Determine the intersection point between an edge and a line. return nil if no intersection
def G6.intersect_edge_line(edge, line)
	pt = Geom.intersect_line_line edge.line, line
	return nil unless pt
	ptbeg = edge.start.position
	return pt if pt == ptbeg
	ptend = edge.end.position
	return pt if pt == ptend
	(pt.vector_to(ptbeg) % pt.vector_to(ptend) < 0) ? pt : nil
end

#detremine if two edges and face for a convex corner at a vertex
def G6.convex_at_vertex(vertex, face, edge1, edge2, face2=nil)
	vother1 = edge1.other_vertex vertex
	vec1 = vertex.position.vector_to vother1.position
	vecin1 = G6.normal_in_to_edge edge1, face
	vecin2 = G6.normal_in_to_edge edge2, ((face2) ? face2 : face)
	((vec1 * vecin1) % (vecin1 * vecin2) >= 0) 
end

#Determine the bissector vector of two edges crossing at a vertex
#Origin of the returned vector is the vertex
def G6.bissector_edges_at_vertex(vertex, edge1, edge2, face)
	vother1 = edge1.other_vertex vertex
	vother2 = edge2.other_vertex vertex
	vec1 = vertex.position.vector_to vother1.position
	vec2 = vertex.position.vector_to vother2.position
	(vec1.parallel?(vec2)) ? vec1 * face.normal : Geom.linear_combination(0.5, vec1, 0.5, vec2) 
end

#Compute the oriented vector of an edge Start --> End
def G6.vector_edge(edge)
	edge.start.position.vector_to edge.end.position
end

#Compute the other edge that is bordering the given face at a given vertex
def G6.other_edge_to_face_at_vertex(vertex, edge, face)
	vertex.edges.each do |e|
		return e if e != edge && (e.faces & [face] == [face])
	end
	nil
end

#Calculate the average center of a list of point
def G6.straight_barycenter(lpt)
	return nil unless lpt && lpt.length > 0
	x = y = z = 0
	lpt.each do |pt|
		x += pt.x
		y += pt.y
		z += pt.z
	end
	n = lpt.length
	Geom::Point3d.new x / n, y / n, z / n
end


end #Module G6

